import Foundation
import UIKit

class CustomUIController : UIViewController, UIPickerViewDelegate, UIPickerViewDataSource
{
    var scannedDevice: String!
    var cardknoxCustomUI: CardknoxSDKCustomUI!
    var timeoutPickerView: UIPickerView!
    var devicePickerView: UIPickerView!
    var commandsTypePickerView: UIPickerView!
    
    var timeoutsDataSource: NSArray = NSArray(array: ["0", "20"])
    var scannedDevices: NSArray!
    
    @IBOutlet weak var timeoutTextField: UITextField!
    @IBOutlet weak var deviceTextField: UITextField!
    @IBOutlet weak var outputTextView: TextViewWithBorder!
    @IBOutlet weak var commandTypeTextField: UITextField!
    @IBOutlet weak var amountTextField: UITextField!
    required init?(coder: NSCoder) {
        super .init(coder: coder)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        let tap = UITapGestureRecognizer(target: self, action: #selector(dismissKeyboard))
        tap.cancelsTouchesInView=true
        view.addGestureRecognizer(tap)
        
        timeoutPickerView = UIPickerView()
        timeoutPickerView.delegate=self
        timeoutPickerView.dataSource=self
        timeoutTextField.inputView=timeoutPickerView
        timeoutPickerView.selectRow(0, inComponent: 0, animated: false)
        self.pickerView(timeoutPickerView, didSelectRow: 0, inComponent: 0)
        
        devicePickerView = UIPickerView();
        devicePickerView.delegate=self
        devicePickerView.dataSource=self
        deviceTextField.inputView = devicePickerView
        self.checkIfScannedDevicesPickerViewEnabled()
        
        commandsTypePickerView=UIPickerView.init()
        commandsTypePickerView.delegate=self
        commandsTypePickerView.dataSource=self
        commandTypeTextField.inputView=commandsTypePickerView
        commandsTypePickerView.selectRow(0, inComponent: 0, animated: false)
        pickerView(commandsTypePickerView, didSelectRow: 0, inComponent: 0)
        
        let notificationCenter = NotificationCenter.default
        
        notificationCenter.addObserver(self, selector: #selector(transactionResultSubscription(aNotification:)), name: Notification.Name( CardknoxSDK.transactionResultSubscription_NSNotificationCenterName()), object: nil)
        
        notificationCenter.addObserver(self, selector: #selector(cardreaderEventSubscription(aNotification:)), name: Notification.Name( CardknoxSDK.cardreaderEventSubscription_NSNotificationCenterName()), object: nil)
        
        notificationCenter.addObserver(self, selector: #selector(customUIScanCompletedSubscription(aNotification:)), name: Notification.Name( CardknoxSDKCustomUI.customUI_scanCompleted_Subscription_NSNotificationCenterName()), object: nil)
        
        notificationCenter.addObserver(self, selector: #selector(customUIScannedDeviceSubscription(aNotification:)), name: Notification.Name( CardknoxSDKCustomUI.customUI_scannedDevice_Subscription_NSNotificationCenterName()), object: nil)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated);
        
        // Documentation purposes
        if(false){
            let notificationCenter = NotificationCenter.default
            
            notificationCenter.addObserver(self, selector: #selector(customUIScanCompletedSubscription(aNotification:)), name: Notification.Name( CardknoxSDKCustomUI.customUI_scanCompleted_Subscription_NSNotificationCenterName()), object: nil)
            
            notificationCenter.addObserver(self, selector: #selector(customUIScannedDeviceSubscription(aNotification:)), name: Notification.Name( CardknoxSDKCustomUI.customUI_scannedDevice_Subscription_NSNotificationCenterName()), object: nil)
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        // RegionStart - InitializeSDK
        CardknoxSDK.setPrincipalKey("Your xKey")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        // RegionEnd - InitializeSDK
        
        CardknoxSDK.setPrincipalKey("ps_demo_cardknox")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        if(cardknoxCustomUI == nil)
        {
            // RegionStart - CreateCardknoxObject
            cardknoxCustomUI = CardknoxSDKCustomUI.create() as? CardknoxSDKCustomUI
            // RegionEnd - CreateCardknoxObject
        }
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        if(cardknoxCustomUI != nil)
        {
            cardknoxCustomUI.destroy()
        }
        cardknoxCustomUI=nil
    }
    
    @IBAction func startScanningButtonClick(_ sender: Any) {
        scannedDevices=nil
        scannedDevice=nil
        deviceTextField.text=nil
        self.checkIfScannedDevicesPickerViewEnabled()
        if(cardknoxCustomUI != nil)
        {
            let timeout = Int32(timeoutTextField.text ?? "") ?? 0
            cardknoxCustomUI.startScanning(withTimeout: timeout)
        }
    }
    
    @IBAction func stopScanningButtonClick(_ sender: Any) {
        if(cardknoxCustomUI != nil)
        {
            cardknoxCustomUI.stopScanning()
        }
    }
    
    @IBAction func connectButtonClick(_ sender: Any) {
        if(scannedDevice != nil)
        {
            if(cardknoxCustomUI != nil)
            {
                // Documentation purposes
                if(false)
                {
                    let exampleDevice : CardknoxSDKCustomUIScannedDevice? = nil;
                    let exampleCustomUI : CardknoxSDKCustomUI? = nil;
                    exampleCustomUI!.connect(withUUID: (exampleDevice!).uuid());
                }
                
                cardknoxCustomUI.connect(withName: scannedDevice)
            }
        }
        else
        {
            ShowAlert(message: "You didn't select device. Select the device you want to connect to.")
        }
    }
    
    @IBAction func disconnectButtonClick(_ sender: Any) {
        if(cardknoxCustomUI != nil)
        {
            cardknoxCustomUI.disconnectFromCurrentDevice()
        }
    }
    
    @IBAction func startTransactionButtonClick(_ sender: Any) {
        if(cardknoxCustomUI != nil)
        {
            let prms = TransactionParameters();
            
            let command = commandTypeTextField.text as NSString?
            prms?.xCommand = command
            
            prms?.vp3300TransactionTimeout = 17;
            
            let amount = ((amountTextField?.text ?? "0") as String).replacingOccurrences(of: ",", with: ".")
            prms?.xAmount = Double(amount) ?? 0
            
            cardknoxCustomUI.startTransaction(withArgs: prms)
        }
    }
    
    @IBAction func stopTransactionButtonClick(_ sender: Any) {
        if(cardknoxCustomUI != nil)
        {
            cardknoxCustomUI.cancelTransaction()
        }
    }
    
    func checkIfScannedDevicesPickerViewEnabled()
    {
        deviceTextField.isEnabled = scannedDevices != nil && scannedDevices.count > 0;
    }
    
    func addLineToOutput(line: String)
    {
        NSLog("----- TEST ----- : %@", line);
        DispatchQueue.main.async {
            self.outputTextView.text.append(contentsOf: line + "\n")
        }
    }
    
    func ShowAlert(message: String?)
    {
        let alert = UIAlertController(title: "Custom UI", message: message, preferredStyle: UIAlertController.Style.alert)
        alert.addAction(UIAlertAction(title: "Ok", style: UIAlertAction.Style.default, handler: nil))
        self.present(alert, animated: true, completion: nil)
    }
    
    @objc func transactionResultSubscription(aNotification: Notification)
    {
        let response = PaymentTransactionResponse.unwrap(aNotification) as? PaymentTransactionResponse

        var message = ""
        if((response?.isSuccess()) != nil)
        {
            let refNum = (response?.xRefNum())! as String
            message = "Transaction success response! Ref num: " + refNum
        }
        else
        {
            let error = (response?.errorMessage())! as String
            message = "Transaction error response - " + error
            
        }
        self.addLineToOutput(line: message)
    }
    
    @objc func cardreaderEventSubscription(aNotification: Notification)
    {
        let callback = CardknoxCardReaderCallback.unwrap(aNotification) as? CardknoxCardReaderCallback
        
        let code = callback?.code()
            
        let name = callback?.name()
        
        addLineToOutput(line: "Card reader - " + name!)
        
        if(code == CardknoxCardReaderCallbackType.error())
        {
            let errorMessage = callback?.message();
            addLineToOutput(line: "Card reader - " + errorMessage!)
        }
    }
    
    @objc func customUIScanCompletedSubscription(aNotification: Notification)
    {
        // Documentation purposes:
        if(false){
            let scanCompleted = CardknoxSDKCustomUIScanCompleted.from(aNotification) as! CardknoxSDKCustomUIScanCompleted;
            let devices : Array<Any> = scanCompleted.scannedDevices();
            var scannedDevice: CardknoxSDKCustomUIScannedDevice? = nil;
            
            // Example to get a first scanned VP3300 device.
            // All VP3300 devices have their internal name start with IDTECH prefix
            if(devices != nil && devices.count > 0)
            {
                for dev in devices{
                    let typedDevice = dev as! CardknoxSDKCustomUIScannedDevice
                    let name = typedDevice.name()!;
                    
                    if name.hasPrefix("IDTECH")
                    {
                        // Save the reference & use it with the "connect to device" method
                        scannedDevice = typedDevice;
                        break;
                    }
                }
            }
        }
        
        let scanCompleted = CardknoxSDKCustomUIScanCompleted.from(aNotification) as? CardknoxSDKCustomUIScanCompleted
        scannedDevices = scanCompleted?.scannedDevices() as NSArray?
        self.checkIfScannedDevicesPickerViewEnabled()
        if(scannedDevices != nil && scannedDevices.count > 0)
        {
            devicePickerView.selectRow(0, inComponent: 0, animated: false)
            self.pickerView(devicePickerView, didSelectRow: 0, inComponent: 0)
        }
    }
    
    // Define a function that accepts a Notification.
    // This method will be invoked when the SDK sends scanned device information
    @objc func customUIScannedDeviceSubscription(aNotification: Notification)
    {
        // Documentation purposes
        if(false){
            // Use the SDK's "response" object utility method to transform a Notification into a "response" object
            let scannedDevice = CardknoxSDKCustomUIScannedDevice.from(aNotification) as! CardknoxSDKCustomUIScannedDevice;
            let name = scannedDevice.name();
            let displayName = scannedDevice.displayName();
            let uuid = scannedDevice.uuid();
            
            // Uuid can be used to connect to the device, like this:
            let exampleCustomUI: CardknoxSDKCustomUI? = nil;
            exampleCustomUI?.connect(withUUID: uuid);
        }
        
        
        let device = CardknoxSDKCustomUIScannedDevice.from(aNotification) as! CardknoxSDKCustomUIScannedDevice
        self.addLineToOutput(line: "Scanned device: " + device.name() + " - " + device.displayName() + " - " + device.uuid());
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if(pickerView == timeoutPickerView)
        {
            return timeoutsDataSource.count
        }
        else if(pickerView == devicePickerView)
        {
            return scannedDevices.count
        }
        else if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands.count
        }
        else
        {
            return 0
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if(pickerView == timeoutPickerView)
        {
            return timeoutsDataSource[row] as? String
        }
        else if(pickerView == devicePickerView)
        {
            let d = scannedDevices[row] as? CardknoxSDKCustomUIScannedDevice
            return d?.name()
        }
        else if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands[row] as? String
        }
        else if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands[row] as? String
        }
        else
        {
            return ""
        }
    }
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        if(pickerView == timeoutPickerView)
        {
            return 1
        }
        else if(pickerView == devicePickerView)
        {
            return 1
        }
        else if(pickerView == commandsTypePickerView)
        {
            return 1
        }
        else
        {
            return 0
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        if(pickerView == timeoutPickerView)
        {
            timeoutTextField.text = timeoutsDataSource[row] as? String
        }
        else if(pickerView == devicePickerView)
        {
            let device = scannedDevices[row] as! CardknoxSDKCustomUIScannedDevice;
            let deviceName : String = device.name();
            scannedDevice = deviceName;
            deviceTextField.text = scannedDevice;
        }
        else if(pickerView == commandsTypePickerView)
        {
            let c = Commands.Commands[row] as? String
            commandTypeTextField.text = c
        }
    }
    
    @objc func dismissKeyboard()
    {
        self.view.endEditing(true)
    }
}
