import Foundation
import UIKit

class InScopeController : UIViewController, UIPickerViewDelegate, UIPickerViewDataSource
{
    var advancedOptionsViewController: AdvancedOptionsController!
    var years: NSMutableArray!
    var months: NSMutableArray!
    var year: String!
    var month: String!
    var expDateSeparator: String = "|"
    var commandsTypePickerView: UIPickerView!
    var expDatePickerView: UIPickerView!
    var cardknoxSDKDirect: CardknoxSDKDirect!
    
    @IBOutlet weak var commandTypeTextField: UITextField!
    @IBOutlet weak var expDateTextField: UITextField!
    @IBOutlet weak var amountTextField: UITextField!
    @IBOutlet weak var cardNumberTextField: UITextField!
    @IBOutlet weak var xTokenTextField: UITextField!
    @IBOutlet weak var outputTextView: TextViewWithBorder!
    @IBOutlet weak var cardNumberContainer: UIView!
    @IBOutlet weak var xTokenContainer: UIView!
    @IBOutlet weak var expDateContainer: UIView!
    @IBOutlet weak var refNumberContainer: UIView!
    @IBOutlet weak var refNumberTextField: UITextField!
    required init?(coder: NSCoder) {
        super .init(coder: coder)
    }
    
    override func viewDidLoad() {
        advancedOptionsViewController = self.storyboard?.instantiateViewController(withIdentifier: "AdvancedOptions") as? AdvancedOptionsController
        
        years = NSMutableArray()
        months = NSMutableArray()
        for i in 0...99
        {
            if(i<10)
            {
                years.add("0"+String(i))
                if(i>0)
                {
                    months.add("0"+String(i))
                }
            }
            else if(i<13)
            {
                years.add(String(i))
                months.add(String(i))
            }
            else
            {
                years.add(String(i))
            }
        }
        month=""
        year=""
        
        commandsTypePickerView=UIPickerView.init()
        commandsTypePickerView.delegate=self
        commandsTypePickerView.dataSource=self
        commandTypeTextField.inputView=commandsTypePickerView
        commandsTypePickerView.selectRow(0, inComponent: 0, animated: false)
        pickerView(commandsTypePickerView, didSelectRow: 0, inComponent: 0)
        
        expDatePickerView=UIPickerView.init()
        expDatePickerView.delegate=self
        expDatePickerView.dataSource=self
        expDateTextField.inputView=expDatePickerView
        let date = Date()
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "yy"
        let indexOfYear = years.index(of: dateFormatter.string(from: date))
        dateFormatter.dateFormat="MM"
        let indexOfMonth = months.index(of: dateFormatter.string(from: date))
        expDatePickerView.selectRow(indexOfMonth, inComponent: 0, animated: false)
        expDatePickerView.selectRow(indexOfYear, inComponent: 1, animated: false)
        pickerView(expDatePickerView, didSelectRow: indexOfMonth, inComponent: 0)
        pickerView(expDatePickerView, didSelectRow: indexOfYear, inComponent: 1)
        
        let tap = UITapGestureRecognizer(target: self, action: #selector(dismissKeyboard))
        tap.cancelsTouchesInView=true
        view.addGestureRecognizer(tap)
    }
    
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        // RegionStart - InitializeSDK
        CardknoxSDK.setPrincipalKey("Your xKey")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        // RegionEnd - InitializeSDK

        
        CardknoxSDK.setPrincipalKey("ps_demo_cardknox")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        
        // RegionStart - CreateCardknoxObject
        cardknoxSDKDirect = CardknoxSDKDirect.create() as? CardknoxSDKDirect;
        // RegionEnd - CreateCardknoxObject
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        
        if(cardknoxSDKDirect != nil)
        {
            // RegionStart - DestroyCardknoxObject
            cardknoxSDKDirect.destroy();
            // RegionEnd - DestroyCardknoxObject
        }
        cardknoxSDKDirect=nil
    }
    
    @IBAction func ExecuteRequest(_ sender: Any)
    {
        self.dismissKeyboard()
        outputTextView.text=""
        // RegionStart - CreateParams
        let prms : TransactionParameters = TransactionParameters.init()
        // RegionEnd - CreateParams
        
        if(false)
        {
            // RegionStart - SetRequiredParams
            prms.xCommand = "<xCommand>";
            prms.xAmount = 1.23;
            // RegionEnd - SetRequiredParams
            
            // RegionStart - SetAdditionalParams
            // Ref num transaction types
            prms.xRefNum = "123456789";
            
            // Card number transaction types
            prms.xCardNum = "4444333322221111";
            prms.xExp = "1225";
            // RegionEnd - SetAdditionalParams
            
            // RegionStart - SetOptionalParams
            prms.xInvoice = "123456";
            prms.xBillFirstName = "Billing first name";
            // ... etc.
            // RegionEnd - SetOptionalParams
        }
          
        let command = commandTypeTextField.text as NSString?
        prms.xCommand=command
        
        let amount = ((amountTextField?.text ?? "0") as String).replacingOccurrences(of: ",", with: ".")
        prms.xAmount = Double(amount) ?? 0
        
        if(Commands.IsRefNum(command: command ?? ""))
        {
            prms.xRefNum = refNumberTextField.text as NSString?
        }
        else
        {
            prms.xToken = xTokenTextField.text as NSString?
            prms.xCardNum = cardNumberTextField.text as NSString?
            
            let expDate = expDateTextField.text?.replacingOccurrences(of: expDateSeparator, with: "")
            prms.xExp = expDate as NSString?
        }
                
        prms.xInvoice = (advancedOptionsViewController.invoiceTextField?.text ?? nil) as NSString?
        prms.xPoNum = (advancedOptionsViewController.poNumberTextField?.text ?? nil) as NSString?
        prms.xDescription = (advancedOptionsViewController.desriptionTextField?.text ?? nil) as NSString?

        prms.xBillFirstName = (advancedOptionsViewController.billFirstNameTextField?.text ?? nil) as NSString?
        prms.xBillLastName = (advancedOptionsViewController.billLastNameTextField?.text ?? nil) as NSString?
        prms.xBillCompany = (advancedOptionsViewController.billCompanyTextField?.text ?? nil) as NSString?
        prms.xBillStreet = (advancedOptionsViewController.billAddressTextField?.text ?? nil) as NSString?
        prms.xBillCity = (advancedOptionsViewController.billCityTextField?.text ?? nil) as NSString?
        prms.xBillZip = (advancedOptionsViewController.billZipTextField?.text ?? nil) as NSString?
        prms.xBillCountry = (advancedOptionsViewController.billCountryTextField?.text ?? nil) as NSString?
        prms.xBillPhone = (advancedOptionsViewController.billPhoneNumberTextField?.text ?? nil) as NSString?

        prms.xShipFirstName = (advancedOptionsViewController.shipFirstNameTextField?.text ?? nil) as NSString?
        prms.xShipLastName = (advancedOptionsViewController.shipLastNameTextField?.text ?? nil) as NSString?
        prms.xShipCompany = (advancedOptionsViewController.shipCompanyTextField?.text ?? nil) as NSString?
        prms.xShipStreet = (advancedOptionsViewController.shipAddressTextField?.text ?? nil) as NSString?
        prms.xShipCity = (advancedOptionsViewController.shipCityTextField?.text ?? nil) as NSString?
        prms.xShipZip = (advancedOptionsViewController.shipZipTextField?.text ?? nil) as NSString?
        prms.xShipCountry = (advancedOptionsViewController.shipCountryTextField?.text ?? nil) as NSString?
        prms.xShipPhone = (advancedOptionsViewController.shipPhoneNumberTextField?.text ?? nil) as NSString?

        prms.xCustom01 = (advancedOptionsViewController.custom01TextField?.text ?? nil) as NSString?
        prms.xCustom02 = (advancedOptionsViewController.custom02TextField?.text ?? nil) as NSString?
        prms.xCustom03 = (advancedOptionsViewController.custom03TextField?.text ?? nil) as NSString?
        
        // RegionStart - CreateRequest
        let request = cardknoxSDKDirect.createRequest(withParameters: prms) as! PaymentTransactionRequestDirect
        // RegionEnd - CreateRequest
        
        var outputText = ""
        if(request.isValid)
        {
            // RegionStart - CallProcess
            let response = request.process() as! PaymentTransactionResponse
            // RegionEnd - CallProcess
            
            if(response.isSuccess())
            {
                // RegionStart - ReadSuccess
                let refNum = response.xRefNum();
                let status = response.xStatus();
                let avs = response.xAvsResult();
                // RegionEnd - ReadSuccess
                outputText = "Transaction successfully created"
            }
            else
            {
                // RegionStart - ReadFailure
                let refNum = response.xRefNum();
                let error = response.xError()!
                let errorCode = response.xErrorCode()!;
                // RegionEnd - ReadFailure
                outputText += "Transaction failed\n"
                outputText += "Error message: " + response.errorMessage()! + "\n"
                outputText += "Error: " + response.xError()! + "\n"
                outputText += "Error code: " + response.xErrorCode()! + "\n"
                outputText += "Ref num: " + response.xRefNum()! + "\n"
            }
        }
        else
        {
            // RegionStart - ReadErrors
            // Request could not be processed due to these errors
            let errors = request.validationErrors!
            // RegionEnd - ReadErrors
            for item in errors
            {
                outputText += item as! String + "\n"
            }
        }
        
        NSLog("----- TEST ----- : %@", outputText);
        outputTextView.text = outputText
    }
    
    @IBAction func OpenAdvancedOptions(_ sender: Any)
    {
        self.present(advancedOptionsViewController, animated: true, completion: nil)
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands.count
        }
        else if(pickerView == expDatePickerView)
        {
            if(component == 0)
            {
                return months.count
            }
            else
            {
                return years.count
            }
        }
        else
        {
            return 0
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands[row] as? String
        }
        else if(pickerView == expDatePickerView)
        {
            if(component == 0)
            {
                return months[row] as? String
            }
            else
            {
                return years[row] as? String
            }
        }
        else
        {
            return ""
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        if(pickerView == commandsTypePickerView)
        {
            let c = Commands.Commands[row] as? String
            commandTypeTextField.text = c
            
            if(Commands.IsRefNum(command: (c ?? "") as NSString))
            {
                refNumberContainer.isHidden = false
                cardNumberContainer.isHidden = true
                expDateContainer.isHidden = true
            }
            else
            {
                refNumberContainer.isHidden = true
                cardNumberContainer.isHidden = false
                expDateContainer.isHidden = false
            }
        }
        else if(pickerView == expDatePickerView)
        {
            if(component == 0)
            {
                month = months[row] as? String
            }
            else
            {
                year = years[row] as? String
            }
            expDateTextField.text=month+expDateSeparator+year
        }
    }
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        if(pickerView == commandsTypePickerView)
        {
            return 1
        }
        else if(pickerView == expDatePickerView)
        {
            return 2
        }
        else
        {
            return 0
        }
    }
    
    @objc func dismissKeyboard()
    {
        self.view.endEditing(true)
    }
}
