import Foundation
import UIKit

class OutOfScopeController : UIViewController, UIPickerViewDelegate, UIPickerViewDataSource
{
    var commandsTypePickerView: UIPickerView!
    var cardknoxSDKUI: CardknoxSDKUI!
    var destroyCardknoxSDKUIOnDisappear: Bool!
    
    @IBOutlet weak var commandTypeTextField: UITextField!
    @IBOutlet weak var showSwipeSwitch: UISwitch!
    @IBOutlet weak var showKeyedSwitch: UISwitch!
    @IBOutlet weak var closeUIOnProcessedTransactionSwitch: UISwitch!
    @IBOutlet weak var amountTextField: UITextField!
    @IBOutlet weak var outputTextView: TextViewWithBorder!
    
    required init?(coder: NSCoder) {
        super .init(coder: coder)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        let tap = UITapGestureRecognizer(target: self, action: #selector(dismissKeyboard))
        tap.cancelsTouchesInView=true
        view.addGestureRecognizer(tap)
        
        commandsTypePickerView=UIPickerView.init()
        commandsTypePickerView.delegate=self
        commandsTypePickerView.dataSource=self
        commandTypeTextField.inputView=commandsTypePickerView
        commandsTypePickerView.selectRow(0, inComponent: 0, animated: false)
        pickerView(commandsTypePickerView, didSelectRow: 0, inComponent: 0)
        
        // RegionStart - SubscribeTransaction
NotificationCenter.default.addObserver(self,
                                       selector: #selector(transactionResultSubscription(aNotification:)),
                                       name: Notification.Name( CardknoxSDK.transactionResultSubscription_NSNotificationCenterName()),
                                       object: nil)
        // RegionEnd - SubscribeTransaction

        
        // RegionStart - SubscribeCardReader
NotificationCenter.default.addObserver(self,
                                       selector: #selector(cardreaderEventSubscription(aNotification:)),
                                       name: Notification.Name( CardknoxSDK.cardreaderEventSubscription_NSNotificationCenterName()),
                                       object: nil)
        // RegionEnd - SubscribeCardReader
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        // RegionStart - InitializeSDK
        CardknoxSDK.setPrincipalKey("Your xKey")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        // RegionEnd - InitializeSDK
        
        
        CardknoxSDK.setPrincipalKey("ps_demo_cardknox")
        CardknoxSDK.setxSoftwareName("Your app name", xSoftwareVersion: "1.0.0", xVersion: "4.5.9")
        if(cardknoxSDKUI == nil)
        {
            // RegionStart - CreateCardknoxObject
            cardknoxSDKUI = CardknoxSDKUI.create() as? CardknoxSDKUI
            // RegionEnd - CreateCardknoxObject
        }
        destroyCardknoxSDKUIOnDisappear=true;
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        
        if(destroyCardknoxSDKUIOnDisappear)
        {
            if(cardknoxSDKUI != nil)
            {
                // RegionStart - DestroyCardknoxObject
                cardknoxSDKUI.destroy()
                // RegionEnd - DestroyCardknoxObject
            }
            cardknoxSDKUI=nil
        }
    }
    
    // RegionStart - SubscribeTransactionMethod
    @objc func transactionResultSubscription(aNotification: Notification)
    {
        let response = PaymentTransactionResponse.unwrap(aNotification) as! PaymentTransactionResponse
        
        var message = ""
        if(response.isSuccess())
        {
            let refNum = response.xRefNum()!
            message = "Transaction success response! Ref num: " + refNum
        }
        else
        {
            let error = response.errorMessage()!
            message = "Transaction error response - " + error
        }
        // RegionEnd - SubscribeTransactionMethod
        
        addLineToOutput(line: message)
        
        // RegionStart - SubscribeTransactionMethod
    }
    // RegionEnd - SubscribeTransactionMethod
    
    
    // RegionStart - SubscribeCardReaderMethod
    @objc func cardreaderEventSubscription(aNotification: Notification)
    {
        let callback = CardknoxCardReaderCallback.unwrap(aNotification) as? CardknoxCardReaderCallback
        
        let code = callback?.code()
        let name = callback?.name()
        
        var errorMessage : String?;
        if(code == CardknoxCardReaderCallbackType.error()){
            errorMessage = callback?.message();
        }
        // RegionEnd - SubscribeCardReaderMethod

        addLineToOutput(line: "Card reader - " + name!)
        if(errorMessage != nil){
            addLineToOutput(line: "Card reader - " + errorMessage!)
        }
        
        // RegionStart - SubscribeCardReaderMethod
    }
    // RegionEnd - SubscribeCardReaderMethod

    func addLineToOutput(line: String)
    {
        NSLog("----- TEST ----- : %@", line);
        DispatchQueue.main.async {
            self.outputTextView.text.append(contentsOf: line + "\n")
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        if(pickerView == commandsTypePickerView)
        {
            let c = Commands.Commands[row] as? String
            commandTypeTextField.text = c
        }
    }
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        if(pickerView == commandsTypePickerView)
        {
            return 1
        }
        else
        {
            return 0
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands[row] as? String
        }
        else
        {
            return ""
        }
    }
    
    @IBAction func showCardknoxUIButtonClick(_ sender: Any)
    {
        dismissKeyboard()
        outputTextView.text=""
        if(cardknoxSDKUI != nil)
        {
            // RegionStart - OutOfScope_GlobalConfig
            // Define if a 'keyed' screen is available
            CardknoxSDKUI.setEnableKeyedEntry(true);
            // Define if a 'swipe' screen is available
            CardknoxSDKUI.setEnableDeviceInsertSwipeTap(true);
            // Define if the UI should auto close
            CardknoxSDKUI.setCloseOnProcessedTransaction(true);
            // RegionEnd - OutOfScope_GlobalConfig
            
            
            CardknoxSDKUI.setEnableKeyedEntry(showKeyedSwitch.isOn)
            CardknoxSDKUI.setEnableDeviceInsertSwipeTap(showSwipeSwitch.isOn)
            CardknoxSDKUI.setCloseOnProcessedTransaction(closeUIOnProcessedTransactionSwitch.isOn)
            
            // RegionStart - CreateParams
            let prms : TransactionParameters = TransactionParameters.init()
            // RegionEnd - CreateParams
            
            // RegionStart - SetRequiredParams
            prms.xCommand = "<xCommand>";
            prms.xAmount = 1.23;
            // RegionEnd - SetRequiredParams
            
            // RegionStart - SetOptionalParams
            prms.xInvoice = "123456";
            prms.xBillFirstName = "Billing first name";
            // ... etc.
            // RegionEnd - SetOptionalParams
            
            
            prms.xCommand=commandTypeTextField.text as NSString?
            
            let amount = ((amountTextField?.text ?? "0") as String).replacingOccurrences(of: ",", with: ".")
            prms.xAmount = Double(amount) ?? 0
            
            // RegionStart - CreateRequest
            let request = cardknoxSDKUI.createRequest(withParameters: prms) as! PaymentTransactionRequestUI
            // RegionEnd - CreateRequest
            
            prms.vp3300TransactionTimeout = 18;

            if(request.isValid)
            {
                destroyCardknoxSDKUIOnDisappear=false
                // RegionStart - CallProcess
                // Show the Cardknox UI
                request.process()
                // RegionEnd - CallProcess
            }
            else
            {
                // RegionStart - ReadErrors
                // Request could not be processed due to these errors
                let errors = request.validationErrors!
                // RegionEnd - ReadErrors
                
                var outputText = ""                
                for item in errors
                {
                    outputText += item as! String + "\n"
                }
                outputTextView.text = outputText
            }
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if(pickerView == commandsTypePickerView)
        {
            return Commands.Commands.count
        }
        else
        {
            return 0
        }
    }
    
    @objc func dismissKeyboard()
    {
        self.view.endEditing(true)
    }
}
